@@ -1,6 +1,6 @@
 /*
 ******************************************************************
-Copyright (c) 2001-2009, Jeff Martin, Tim Bacon
+Copyright (c) 2001-2013, Jeff Martin, Tim Bacon
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -703,6 +703,18 @@ public class DifferenceEngine implements DifferenceConstants {
                 control, test, listener, d);
     }
 
+    /**
+     * @param attr
+     * @return true if the attribute is an XML Schema Instance
+     * namespace attribute XMLUnit treats in a special way.
+     */
+    private boolean isXMLSchemaTypeAttribute(Attr attr) {
+        return XMLConstants
+            .W3C_XML_SCHEMA_INSTANCE_NS_URI.equals(attr.getNamespaceURI())
+            && XMLConstants.W3C_XML_SCHEMA_INSTANCE_TYPE_ATTR
+                .equals(attr.getLocalName());
+    }
+
     /**
      * Compare two attributes
      * @param control
@@ -717,15 +729,44 @@ public class DifferenceEngine implements DifferenceConstants {
         
         compare(control.getPrefix(), test.getPrefix(), control, test, 
                 listener, NAMESPACE_PREFIX);
-                
-        compare(control.getValue(), test.getValue(), control, test,
-                listener, ATTR_VALUE);
-
+        if (isXMLSchemaTypeAttribute(control)
+            && isXMLSchemaTypeAttribute(test)) {
+            compareXMLSchemaTypeAttributeValues(control, test, listener);
+        } else {
+            compare(control.getValue(), test.getValue(), control, test,
+                    listener, ATTR_VALUE);
+        }
         compare(control.getSpecified() ? Boolean.TRUE : Boolean.FALSE,
                 test.getSpecified() ? Boolean.TRUE : Boolean.FALSE,
                 control, test, listener, ATTR_VALUE_EXPLICITLY_SPECIFIED);
     }
 
+    private void compareXMLSchemaTypeAttributeValues(Attr control, Attr test,
+                                                     DifferenceListener listener)
+        throws DifferenceFoundException {
+        String controlValue = control.getValue();
+        String testValue = test.getValue();
+
+        String controlLocal = controlValue;
+        String controlPrefix = "";
+        String testLocal = testValue;
+        String testPrefix = "";
+        int controlColon = controlValue.indexOf(":");
+        if (controlColon >= 0 && controlColon < controlValue.length() - 1) {
+            controlLocal = controlValue.substring(controlColon + 1);
+            controlPrefix = controlValue.substring(0, controlColon);
+        }
+        int testColon = testValue.indexOf(":");
+        if (testColon >= 0 && testColon < testValue.length() - 1) {
+            testLocal = testValue.substring(testColon + 1);
+            testPrefix = testValue.substring(0, testColon);
+        }
+        compare("{" + findNamespaceURIForPrefix(control, controlPrefix) + "}"
+                + controlLocal,
+                "{" + findNamespaceURIForPrefix(test, testPrefix) + "}"
+                + testLocal, control, test, listener, ATTR_VALUE);
+    }
+
     /**
      * Compare two CDATA sections - unused, kept for backwards compatibility
      * @param control
@@ -940,6 +981,34 @@ public class DifferenceEngine implements DifferenceConstants {
         return changed ? sb.toString() : orig;
     }
 
+    /**
+     * Try to find the namespace URI that is mapped to the given
+     * prefix at the given node.
+     */
+    private String findNamespaceURIForPrefix(Node onNode, String prefix) {
+        if (onNode != null && onNode instanceof Attr) {
+            onNode = ((Attr) onNode).getOwnerElement();
+        }
+        while (onNode != null && onNode.getNodeType() != Node.ELEMENT_NODE) {
+            onNode = onNode.getParentNode();
+        }
+        if (onNode == null) {
+            return null;
+        }
+
+        NamedNodeMap attrs = onNode.getAttributes();
+        Attr attr = null;
+        if (prefix == null || "".equals(prefix)) {
+            attr = (Attr) attrs.getNamedItem(XMLConstants.XMLNS_PREFIX);
+        } else {
+            attr = (Attr) attrs.getNamedItemNS(XMLConstants.XMLNS_ATTRIBUTE_URI, prefix);
+        }
+        if (attr != null) {
+            return attr.getValue();
+        }
+        return findNamespaceURIForPrefix(onNode.getParentNode(), prefix);
+    }
+
     /**
      * Marker exception thrown by the protected compare() method and passed
      * upwards through the call stack to the public compare() method.
